#ifndef __TITAN3C_H
#define __TITAN3C_H
#include "orbitersdk.h"
#include "multi_stage.h"
#include "x20.h"


const VECTOR3 OFS_STAGE3       = { 0.0, 0.0, -3.05-2.25};
const VECTOR3 OFS_STAGE2       = { 0.0, 0.0, OFS_STAGE3.z-2.25-3.85};
const VECTOR3 OFS_STAGE1       = { 0.0, 0.0, OFS_STAGE2.z-2.5-11.5};
const VECTOR3 OFS_STAGE0_LEFT  = { -1.55-1.5, 0.0, OFS_STAGE1.z+1.5};
const VECTOR3 OFS_STAGE0_RIGHT = { 1.55+1.5 , 0.0, OFS_STAGE1.z+1.5};

const VECTOR3 OFS_STAGE0_CENTER1_FLAME = { 0.0, 0.0,  OFS_STAGE1.z-11.5};
const VECTOR3 OFS_STAGE0_CENTER2_FLAME = { 0.0, 0.0,  OFS_STAGE1.z-11.5};
const VECTOR3 OFS_STAGE0_LEFT_FLAME = { -1.55-1.5, 0.0,  OFS_STAGE1.z-11.5};
const VECTOR3 OFS_STAGE0_RIGHT_FLAME = {1.55+1.5, 0.0,  OFS_STAGE1.z-11.5};
const VECTOR3 OFS_STAGE1_FLAME = { 0.0, 0.0,  OFS_STAGE1.z-11.5};
const VECTOR3 OFS_STAGE2_FLAME = { 0.0, 0.0,  OFS_STAGE2.z-4.3};
const VECTOR3 OFS_STAGE3_FLAME = { 0.0, 0.0,  OFS_STAGE3.z-2.25};


const double MASS_THIRD    = 12247*KG;
const double MASS_SECOND   = 29188*KG;
const double MASS_FIRST    = 116573*KG;
const double MASS_BOOSTER  = 226233*KG;

const double MASS_THIRD_EMPTY   = 1950*KG;
const double MASS_THIRD_FUEL    = MASS_THIRD-MASS_THIRD_EMPTY;
const double MASS_SECOND_EMPTY  = 2653*KG;
const double MASS_SECOND_FUEL   = MASS_SECOND-MASS_SECOND_EMPTY;
const double MASS_FIRST_EMPTY   = 5443*KG;
const double MASS_FIRST_FUEL    = MASS_FIRST-MASS_FIRST_EMPTY;
const double MASS_BOOSTER_EMPTY = 33798*KG;
const double MASS_BOOSTER_FUEL  = MASS_BOOSTER-MASS_BOOSTER_EMPTY;


const double ISP_BOOSTER_SL  = 238*SEC;
const double ISP_BOOSTER_VAC = 263*SEC;
const double ISP_FIRST_SL    = 250*SEC;
const double ISP_FIRST_VAC   = 302*SEC;
const double ISP_SECOND_SL   = 145*SEC;
const double ISP_SECOND_VAC  = 358*SEC;
const double ISP_THIRD_SL    =   0*SEC;
const double ISP_THIRD_VAC   = 311*SEC;

const double THRUST_BOOSTER_VAC = 596474*KGF;
const double THRUST_FIRST_VAC   = 238590*KGF;
const double THRUST_SECOND_VAC  =  46266*KGF;
const double THRUST_THIRD_VAC   =   7257*KGF;



void AddLeftBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("tit3_stg0l", &OFS_STAGE0_LEFT);
};


void AddRightBoosterStage(VESSEL *vessel)
{
	vessel->AddMesh ("tit3_stg0r", &OFS_STAGE0_RIGHT);
};


void AddFirstStage(VESSEL *vessel)
{
	vessel->AddMesh ("tit3_stg1", &OFS_STAGE1);
};

void AddSecondStage(VESSEL *vessel)
{
	vessel->AddMesh ("tit3_stg2", &OFS_STAGE2);
};

void AddThirdStage(VESSEL *vessel)
{
	vessel->AddMesh ("tit3_stg3", &OFS_STAGE3);
};


class Titan3cThirdStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void JettisonX20(VESSEL *vessel);
	bool bManualSeparate;
	double slThrust;
	double vacThrust;
	double stage_sep;
};

void Titan3cThirdStage::Init ()
{
	bManualSeparate = false;
	slThrust  = CalcSeaLevelThrust(THRUST_THIRD_VAC,ISP_THIRD_VAC,ISP_THIRD_SL,MASS_THIRD_FUEL);
	vacThrust = THRUST_THIRD_VAC;
	stage_sep=0;
};

void Titan3cThirdStage::Exit (VESSEL *vessel)
{
};

void Titan3cThirdStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	stage_sep=stage_sep_time;
	SetStage(vessel);
};

void Titan3cThirdStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	vessel->ParseScenarioLine (line, def_vs);
};

void Titan3cThirdStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 3);
};

void Titan3cThirdStage::Timestep (VESSEL *vessel, double simt)
{
	IStage *newStage;
	if (bManualSeparate)
	{
		bManualSeparate=false;
		JettisonStage(vessel);
		newStage = new X20_Orbit();
		Stack->SetStage(newStage,vessel,simt);
		return;
	}
	DoThrustAdjust(vessel,slThrust,vacThrust);
};

void Titan3cThirdStage::Animate (VESSEL *vessel, double simt)
{
};

void Titan3cThirdStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};

void Titan3cThirdStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Titan3cThirdStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
	}
	return 0;
};

void Titan3cThirdStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (10.0);
	vessel->SetCOG_elev (6.75);
	vessel->SetMaxFuelMass(MASS_THIRD_FUEL);
	vessel->SetFuelMass   (MASS_THIRD_FUEL);
	vessel->SetEmptyMass  (MASS_THIRD_EMPTY+MASS_X20);
	vessel->SetISP        (ISP_THIRD_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e3);
	//vessel->SetPMI (_V(18.5+3.5,24.75+3.5,10.0+.85));
	//vessel->SetCrossSections (_V(40.25+12.5,62.75+12.5,19.75));
	vessel->SetCW (0.09, 0.09, 2, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE3_FLAME ;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	AddThirdStage(vessel);
	AddX20(vessel);
};

void Titan3cThirdStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_STAGE3;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_Third");
	VESSEL::Create (name, "Titan3C_Stage_Third", vs);
};


class Titan3cSecondStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void JettisonX20(VESSEL *vessel);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
};

void Titan3cSecondStage::Init ()
{
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
	slThrust        = CalcSeaLevelThrust(THRUST_SECOND_VAC,ISP_SECOND_VAC,ISP_SECOND_SL,MASS_SECOND_FUEL);
	vacThrust       = THRUST_SECOND_VAC;
};
void Titan3cSecondStage::Exit (VESSEL *vessel)
{
};
void Titan3cSecondStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};
void Titan3cSecondStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	vessel->ParseScenarioLine (line, def_vs);
};
void Titan3cSecondStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 2);
};
void Titan3cSecondStage::Timestep (VESSEL *vessel, double simt)
{
	IStage *newStage;
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		VESSELSTATUS vs;
		double velocity;
		vessel->GetStatus(vs);
		velocity = (vs.rvel.x * vs.rvel.x +vs.rvel.y * vs.rvel.y+vs.rvel.z * vs.rvel.z);
		if (velocity > (6330 * 6330) && vessel->GetAtmPressure() < 5000)
		{
			//Mode III
		}
		else if (vessel->GetAtmPressure()<5000)
		{
			//Mode II
		}
		else if (vessel->GetAtmPressure()<56000)
		{
			//Delayed Mode II
		}
		else if (vessel->GetAtmPressure()>=56000)
		{
			//Mode I
	//		JettisonGemini(vessel);
	//		newStage = new Gemini();
	//		Stack->SetStage(newStage,vessel,simt);
		}
		return;
	}
	else 
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			bManualSeparate=false;
			JettisonStage(vessel);
			newStage = new Titan3cThirdStage();
			Stack->SetStage(newStage,vessel,simt);
			vessel->SetEngineLevel(ENGINE_MAIN, 1);
			return;
		}
	}
	DoThrustAdjust(vessel,slThrust,vacThrust);
	//vessel->SetMaxThrust(ENGINE_MAIN,GetCurrentValue(vessel,slThrust,vacThrust));
};
void Titan3cSecondStage::Animate (VESSEL *vessel, double simt)
{
};
void Titan3cSecondStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Titan3cSecondStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Titan3cSecondStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Titan3cSecondStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_SECOND_FUEL);
	vessel->SetFuelMass(MASS_SECOND_FUEL);
	vessel->SetEmptyMass  (MASS_SECOND_EMPTY + MASS_THIRD + MASS_X20);
	vessel->SetISP        (ISP_SECOND_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e4);
//	vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
//	vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE2_FLAME.z), _V(-.7,.7,OFS_STAGE2_FLAME.z), _V(.7,.7,OFS_STAGE2_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE2_FLAME ;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	AddSecondStage(vessel);
	AddThirdStage(vessel);
	AddX20(vessel);
};

void Titan3cSecondStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_STAGE2;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_Second");
	VESSEL::Create (name, "Titan3C_Stage_Second", vs);
};


void Titan3cSecondStage::JettisonX20(VESSEL *vessel)
{

};


class Titan3cFirstStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void JettisonX20(VESSEL *vessel);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
};

void Titan3cFirstStage::Init ()
{
	bManualSeparate = false;
	bAbort=false;
	doAbort=false;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
};
void Titan3cFirstStage::Exit (VESSEL *vessel)
{
};
void Titan3cFirstStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};
void Titan3cFirstStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	vessel->ParseScenarioLine (line, def_vs);
};
void Titan3cFirstStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 1);
};
void Titan3cFirstStage::Timestep (VESSEL *vessel, double simt)
{
	IStage *newStage;
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		VESSELSTATUS vs;
		double velocity;
		vessel->GetStatus(vs);
		velocity = (vs.rvel.x * vs.rvel.x +vs.rvel.y * vs.rvel.y+vs.rvel.z * vs.rvel.z);
		if (velocity > (6330 * 6330) && vessel->GetAtmPressure() < 5000)
		{
			//Mode III
		}
		else if (vessel->GetAtmPressure()<5000)
		{
			//Mode II
		}
		else if (vessel->GetAtmPressure()<56000)
		{
			//Delayed Mode II
		}
		else if (vessel->GetAtmPressure()>=56000)
		{
		}
		doAbort=false;
		return;
	}
	else 
	{
		if (vessel->GetFuelMass() == 0 || bManualSeparate)
		{
			bManualSeparate=false;
			JettisonStage(vessel);
			newStage = new Titan3cSecondStage();
			Stack->SetStage(newStage,vessel,simt);
			vessel->SetEngineLevel(ENGINE_MAIN, 1);
			return;
		}
	}
	DoThrustAdjust(vessel,slThrust,vacThrust);
};
void Titan3cFirstStage::Animate (VESSEL *vessel, double simt)
{
};
void Titan3cFirstStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Titan3cFirstStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Titan3cFirstStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Titan3cFirstStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(MASS_FIRST_FUEL);
	vessel->SetFuelMass(MASS_FIRST_FUEL);
	vessel->SetEmptyMass  (MASS_FIRST_EMPTY + MASS_SECOND + MASS_THIRD + MASS_X20);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN, vacThrust);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e4);
//	vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
//	vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z-40.0), _V(-.7,.7,OFS_STAGE1_FLAME.z-40.0), _V(.7,.7,OFS_STAGE1_FLAME.z-40.0));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE1_FLAME ;
	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	AddFirstStage(vessel);
	AddSecondStage(vessel);
	AddThirdStage(vessel);
	AddX20(vessel);
};

void Titan3cFirstStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-2.0);
	ofs = OFS_STAGE1;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_First");
	VESSEL::Create (name, "Titan3C_Stage_First", vs);
};

void Titan3cFirstStage::JettisonX20(VESSEL *vessel)
{

};

class Titan3cBoosterStage:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	void JettisonX20(VESSEL *vessel);
	void doBoosterTimeStep (VESSEL *vessel, double simt);
	bool bManualSeparate;
	bool bAbort;
	bool doAbort;
	double slThrust;
	double vacThrust;
	double slBooster;
	double vacBooster;
	double last_simt;
	double booster_fuel;
	double empty_mass;
	double booster_ignition_time;
	bool booster_ignited;
	double booster_sep_time;
	int boost_id1;
	int boost_id2;
};

void Titan3cBoosterStage::Init ()
{
	bManualSeparate = false;
	bAbort          = false;
	doAbort         = false;
	slThrust        = CalcSeaLevelThrust(THRUST_FIRST_VAC,ISP_FIRST_VAC,ISP_FIRST_SL,MASS_FIRST_FUEL);
	vacThrust       = THRUST_FIRST_VAC;
	slBooster       = CalcSeaLevelThrust(THRUST_BOOSTER_VAC,ISP_BOOSTER_VAC,ISP_BOOSTER_SL,MASS_BOOSTER_FUEL);
	vacBooster      = THRUST_BOOSTER_VAC;
	last_simt       = 0.0;
	booster_fuel    = MASS_BOOSTER_FUEL;
	empty_mass      = MASS_BOOSTER_EMPTY*2 + MASS_FIRST + MASS_SECOND + MASS_THIRD + MASS_X20;
	booster_sep_time      = 115;
	booster_ignition_time = 0;
	booster_ignited       = false;
	Booster_Setup(booster_sep_time,MASS_BOOSTER_FUEL);
};

void Titan3cBoosterStage::Exit (VESSEL *vessel)
{
};

void Titan3cBoosterStage::Setup (VESSEL *vessel, double stage_sep_time)
{
	SetStage(vessel);
};

void Titan3cBoosterStage::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	double old_ignition_time=0.0;
    if (!strnicmp (line, "ITIME", 5)) {
        sscanf (line+5, "%lf", &old_ignition_time);
		booster_ignition_time=-old_ignition_time;
    } else {
        vessel->ParseScenarioLine (line, def_vs);
    }
};

void Titan3cBoosterStage::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 0);
	oapiWriteScenario_float (scn, "ITIME", last_simt-booster_ignition_time);
};
void Titan3cBoosterStage::Timestep (VESSEL *vessel, double simt)
{
	double FuelLevel=0.0;
	double booster_time=0.0;
	IStage *newStage;
	
	if (bAbort)
	{
		vessel->SetEngineLevel(ENGINE_MAIN,1.0);				
		bAbort = false;
		doAbort = true;
	}
	else if (doAbort)
	{
		VESSELSTATUS vs;
		double velocity;
		vessel->GetStatus(vs);
		velocity = (vs.rvel.x * vs.rvel.x +vs.rvel.y * vs.rvel.y+vs.rvel.z * vs.rvel.z);
		if (velocity > (6330 * 6330) && vessel->GetAtmPressure() < 5000)
		{
			//Mode III
		}
		else if (vessel->GetAtmPressure()<5000)
		{
			//Mode II
		}
		else if (vessel->GetAtmPressure()<56000)
		{
			//Delayed Mode II
		}
		else if (vessel->GetAtmPressure()>=56000)
		{

		}
		doAbort=false;
		return;
	}
	else 
	{
		if (!booster_ignited)
		{
			if (vessel->GetEngineLevel (ENGINE_MAIN) > 0.95) 
			{
				booster_ignited = true;
				booster_ignition_time = simt; 
			}
		}
		else
		{
			if (((simt-booster_ignition_time) > booster_sep_time) || bManualSeparate)
			{
				bManualSeparate=false;
				JettisonStage(vessel);
				newStage = new Titan3cFirstStage();
				Stack->SetStage(newStage,vessel,simt);
				vessel->SetEngineLevel(ENGINE_MAIN,1.0);
				return;
			};
		}
	}
	doBoosterTimeStep(vessel,simt);
	last_simt=simt;
};

void Titan3cBoosterStage::doBoosterTimeStep (VESSEL *vessel, double simt)
{
	double *booster_thrust;
	double c_booster_thrust;
	double thrust_level, prop_level;
	double booster_time=0.0;
	booster_thrust=vessel->GetMainThrustModPtr();	
	*booster_thrust = 0.0;
	if (booster_ignited)
	{
		booster_time = simt-booster_ignition_time;
		booster_thrust = vessel->GetMainThrustModPtr();
		GetBooster_State (booster_time, thrust_level, prop_level);
		c_booster_thrust = GetCurrentValue(vessel,slBooster,vacBooster)*2*thrust_level;
		vessel->SetEmptyMass(empty_mass+(booster_fuel*2*prop_level));
		double exhaust_scale_1 = 16.0*thrust_level;
		double exhaust_scale_2 = 2.0*sqrt(thrust_level);
		vessel->SetExhaustScales (EXHAUST_CUSTOM, boost_id1, 10, 1);
		vessel->SetExhaustScales (EXHAUST_CUSTOM, boost_id2, 10, 1);
	}
	else
	{
		c_booster_thrust = 0.0;
	}
	*booster_thrust = (GetCurrentValue(vessel,slThrust,vacThrust)-vacThrust)+c_booster_thrust;
}

void Titan3cBoosterStage::Animate (VESSEL *vessel, double simt)
{
};
void Titan3cBoosterStage::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void Titan3cBoosterStage::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int Titan3cBoosterStage::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "Abort Emergency"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				bAbort = true;
			return 1;
		}
	}
	return 0;
};

void Titan3cBoosterStage::SetStage(VESSEL *vessel)
{
	vessel->SetSize (15.0);
	vessel->SetCOG_elev (13);
	vessel->SetMaxFuelMass(1);
	vessel->SetEmptyMass  (empty_mass + booster_fuel*2);
	vessel->SetISP        (ISP_FIRST_VAC);
	vessel->SetMaxThrust  (ENGINE_MAIN,  1);
	vessel->SetMaxThrust  (ENGINE_RETRO, 0);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 5e4);

	//vessel->SetPMI (_V(40+18.5+3.5,40+24.75+3.5,1.25+10.0+.85));
	//vessel->SetCrossSections (_V(70+40.25+12.5,70+62.75+12.5,19.75));
	vessel->SetTouchdownPoints (_V(0,-1.0,OFS_STAGE1_FLAME.z), _V(-.7,.7,OFS_STAGE1_FLAME.z), _V(.7,.7,OFS_STAGE1_FLAME.z));
	vessel->SetCW (0.09, 0.09, 2.0, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 mesh_dir;
	VECTOR3 mesh_pos;
	mesh_dir=_V(0,0,-1);
	mesh_pos=OFS_STAGE1_FLAME ;
//	vessel->AddExhaustRef(EXHAUST_MAIN, mesh_pos,10,1,&mesh_dir);
	boost_id1 = vessel->AddExhaustRef (EXHAUST_CUSTOM,_V(OFS_STAGE0_LEFT_FLAME.x,OFS_STAGE0_LEFT_FLAME.y,OFS_STAGE0_LEFT_FLAME.z), 0, 0);
	boost_id2 = vessel->AddExhaustRef (EXHAUST_CUSTOM,_V(OFS_STAGE0_RIGHT_FLAME.x,OFS_STAGE0_RIGHT_FLAME.y,OFS_STAGE0_RIGHT_FLAME.z), 0, 0);
	AddLeftBoosterStage(vessel);
	AddRightBoosterStage(vessel);
	AddFirstStage(vessel);
	AddSecondStage(vessel);
	AddThirdStage(vessel);
	AddX20(vessel);
};

void Titan3cBoosterStage::JettisonStage(VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(-2.0,0,0);
	ofs = OFS_STAGE0_LEFT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_LeftBooster");
	VESSEL::Create (name, "Titan3c_Stage_LeftBooster", vs);

	vel = _V(2.0,0,0);
	ofs = OFS_STAGE0_RIGHT;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_RightBooster");
	VESSEL::Create (name, "Titan3c_Stage_RightBooster", vs);
};


void Titan3cBoosterStage::JettisonX20(VESSEL *vessel)
{

};


#endif