#ifndef __X20_H
#define __X20_H

#include "orbitersdk.h"
#include "multi_stage.h"
#include <stdio.h>
#include <fstream.h>
#include <math.h>

const VECTOR3 OFS_X20 = { 0.0, 0.0, 0.0};
const double MASS_X20_EMPTY  = 5165*KG+300*KG;
const double MASS_X20_FUEL   = 1060*KG;
const double MASS_X20        = MASS_X20_FUEL + MASS_X20_EMPTY;

double LiftCoeff (double aoa)
{
	const int nlift = 9;
	static const double AOA[nlift] = {-180*RAD,-60*RAD,-30*RAD,-1*RAD,15*RAD,20*RAD,25*RAD,60*RAD,180*RAD};
	static const double CL[nlift]  = {       0,      0,   -0.2,     0,   0.8,     1,   0.8,     0,      0};
	static const double SCL[nlift] = {(CL[1]-CL[0])/(AOA[1]-AOA[0]), (CL[2]-CL[1])/(AOA[2]-AOA[1]),
		                              (CL[3]-CL[2])/(AOA[3]-AOA[2]), (CL[4]-CL[3])/(AOA[4]-AOA[3]),
									  (CL[5]-CL[4])/(AOA[5]-AOA[4]), (CL[6]-CL[5])/(AOA[6]-AOA[5]),
									  (CL[7]-CL[6])/(AOA[7]-AOA[6]), (CL[8]-CL[7])/(AOA[8]-AOA[7])};
	for (int i = 0; i < nlift-1 && AOA[i+1] < aoa; i++);
	return CL[i] + (aoa-AOA[i])*SCL[i];
}



void AddAttitudeJets(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	const double ATTCOOR = .4;
	const double ATTZ = 2.0;
	const double TRANCOOR = .75;
	const double TRANZ=-.65;
	const double ATTWIDTH=.1;
	const double ATTHEIGHT=.5;
	const double TRANWIDTH=.2;
	const double TRANHEIGHT=1;
	const double RCSOFFSET=.1;

	//up
	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);


	//dn
	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);


	//right
	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);

	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);

	//left
	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);

	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	//rot right
	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_ref= _V(ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_ref= _V(-ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	//rot left
	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_ref= _V(-ATTCOOR,ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_ref= _V(ATTCOOR,-ATTCOOR,ATTZ-RCSOFFSET);
	att_dir = _V(0,-1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,ATTWIDTH,ATTHEIGHT);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

}

void AddX20(VESSEL *vessel)
{
	vessel->AddMesh ("x20", &OFS_X20);
};


class X20_Orbit:public IStage
{
public:
	void Init ();
	void Exit (VESSEL *vessel);
	void Setup (VESSEL *vessel, double stage_sep_time);
	void ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs);
	void SaveState (VESSEL *vessel, FILEHANDLE scn);
	void Timestep (VESSEL *vessel, double simt);
	void Animate (VESSEL *vessel, double simt);
	void VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount);
	void VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount);
	int ConsumeKey (VESSEL *vessel, const char *keystate);
private:
	void SetStage(VESSEL *vessel);
	void JettisonStage(VESSEL *vessel);
	bool bManualSeparate;
	double stage_sep;
};

void X20_Orbit::Init ()
{
	bManualSeparate = false;
	stage_sep=0;
};

void X20_Orbit::Exit (VESSEL *vessel)
{
};

void X20_Orbit::Setup (VESSEL *vessel, double stage_sep_time)
{
	stage_sep=stage_sep_time;
	SetStage(vessel);
};

void X20_Orbit::ParseLine (VESSEL *vessel, char *line, VESSELSTATUS *def_vs)
{
	vessel->ParseScenarioLine (line, def_vs);
};

void X20_Orbit::SaveState (VESSEL *vessel, FILEHANDLE scn)
{
	oapiWriteScenario_int (scn, "CONFIGURATION", 4);
};

void X20_Orbit::Timestep (VESSEL *vessel, double simt)
{

};

void X20_Orbit::Animate (VESSEL *vessel, double simt)
{
};
void X20_Orbit::VisualCreated (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
void X20_Orbit::VisualDestroyed (VESSEL *vessel, VISHANDLE vis, int refcount)
{
};
int X20_Orbit::ConsumeKey (VESSEL *vessel, const char *keystate)
{
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bManualSeparate = true;
			return 1;
		}

	}
	return 0;
};

void X20_Orbit::SetStage(VESSEL *vessel)
{
	vessel->SetSize (10.0);
	vessel->SetCOG_elev (3.0);
	vessel->SetCameraOffset(_V(0,2,5.8));
	vessel->SetMaxFuelMass(MASS_X20_FUEL);
	vessel->SetFuelMass   (MASS_X20_FUEL);
	vessel->SetEmptyMass  (MASS_X20_EMPTY);
	vessel->SetISP        (265*SEC);
	vessel->SetMaxThrust  (ENGINE_MAIN, 18140*KGF);
	vessel->SetMaxThrust  (ENGINE_RETRO, 50* KGF);
	vessel->SetMaxThrust  (ENGINE_HOVER, 0);
	vessel->SetMaxThrust  (ENGINE_ATTITUDE, 25*KGF);
	vessel->SetDockParams(_V(0,1.5,0),_V(0,1,0),_V(0,0,-1));
	vessel->SetPMI (_V(18.5,24.75,10.0));
	vessel->SetCrossSections (_V(40.25,62.75,19.75));
	vessel->SetCW (.09, .09, 2, 1.4);
	vessel->SetRotDrag (_V(2.0,2.0,2.0));
	vessel->SetPitchMomentScale (1e-5);
	vessel->SetBankMomentScale (2e-5);
	vessel->SetLiftCoeffFunc (LiftCoeff); 
	vessel->SetTouchdownPoints (_V(0,-4,17), _V(-3.96,-5,-4.3), _V(3.96,-5,-4.3)); 
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	m_exhaust_pos= _V(0,0.75,-5);
	m_exhaust_ref = _V(0,0,-1);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 2.0, .1, &m_exhaust_ref);

	m_exhaust_pos= _V(0,-1.75,5.5);
	m_exhaust_ref = _V(0,0,1);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 1.7,.085,  &m_exhaust_ref);
	AddX20(vessel);
};


void X20_Orbit::JettisonStage(VESSEL *vessel)
{
/*	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,-0.5);
	ofs = OFS_MID;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;
	vs.vrot.y = 0.0;
	vs.vrot.z = 0.0;
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Stage_Second");
	VESSEL::Create (name, "Titan_Stage_Second", vs);
*/
};



#endif