// ==============================================================
// ORBITER vessel module: Mercury Redstone
// ==============================================================

#include "orbitersdk.h"
#include "multi_stage.h"
#include "x20.h"
#include "titan3c.h"


// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	Stack = new Multi_Stage();
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (Stack) delete Stack;
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	Stack->ConsumeKey(vessel,keystate);
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	IStage *newStage;
	newStage = new Titan3cBoosterStage();
	//newStage = new Titan3cFirstStage();
	Stack->SetStage(newStage,vessel,0);

}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	Stack->Timestep(vessel,simt);
}

DLLCLBK void ovcLoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
	IStage *newStage;
    char *line;
	int status = 0;
	bool configurationset = false;
	while (oapiReadScenario_nextline (scn, line)) {
		if (configurationset)
		{
			Stack->ParseLine(vessel,line,def_vs);
		}
		else
		{
			if (!strnicmp (line, "CONFIGURATION", 13)) {
				sscanf (line+13, "%d", &status);
				switch (status) {
				case 0:
					newStage = new Titan3cBoosterStage();
				case 1:
					newStage = new Titan3cFirstStage();
					break;
				case 2:
					newStage = new Titan3cSecondStage();
					break;
				case 3:
					newStage = new Titan3cThirdStage();
					break;
				case 4:
					newStage = new X20_Orbit();
					break;
				}
				Stack->SetStage(newStage,vessel,0);
				configurationset = true;
			} else {
				vessel->ParseScenarioLine (line, def_vs);
			}
		}
    }
//	Stack->LoadState(vessel,scn,def_vs);
}
//

// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	vessel->SaveDefaultState(scn);
	Stack->SaveState(vessel,scn);
}
